using System.Collections.Generic;
using UnityEngine;


public class ProjectileHandler
{
    public GameObject _projectileObject;
    public Rigidbody _rigidBody;
    public float _activationTime;

    public ProjectileHandler(GameObject projectileObject)
    {
        _projectileObject = projectileObject;
        _rigidBody = projectileObject.GetComponent<Rigidbody>();
        _activationTime = 0.0f;
    }
}


public class AICannonHandler : MonoBehaviour
{
    [Header("AI Cannon Handler properties")]
    [Tooltip("The target")]
    public GameObject _targetObject;

    [Tooltip("The projectile prefab")]
    public GameObject _projectilePrefab;

    [Tooltip("The projectiles start transform reference")]
    public Transform _projectileStartTransform;

    [Tooltip("The size of the projectile pool")]
    public int _projectilesInPool = 10;

    [Tooltip("Projectile time to live.")]
    public float _projectileTimeToLive = 2.0f;

    [Tooltip("Projectile launch speed.")]
    public float _projectileLaunchSpeed = 10.0f;

    [Tooltip("How long to wait between each shot.")]
    public float _reloadTime = 2.0f;

    private Rigidbody _targetRigidbody;
    private Transform _targetTransform;
    private List<ProjectileHandler> _projectiles = new List<ProjectileHandler>();
    private float _lastFiredTime = 0.0f;
    private float _projectileMass;


    private void Start()
    {
        _targetRigidbody = _targetObject.GetComponent<Rigidbody>();
        _targetTransform = _targetObject.transform;
        _projectileMass = _projectilePrefab.GetComponent<Rigidbody>().mass;

        // Create projectile pool
        for (int i = 0; i < _projectilesInPool; i++)
        {
            GameObject projectileObject = Instantiate(_projectilePrefab);
            projectileObject.SetActive(false);
            ProjectileHandler projectileHandler = new ProjectileHandler(projectileObject);
            _projectiles.Add(projectileHandler);
        }
    }


    private ProjectileHandler GetAvailableProjectile()
    {
        for (int i = 0; i < _projectilesInPool; i++)
        {
            ProjectileHandler projectileHandler = _projectiles[i];
            if (!projectileHandler._projectileObject.activeSelf)
            {
                projectileHandler._projectileObject.SetActive(true);
                projectileHandler._activationTime = Time.time;
                return projectileHandler;
            }
        }
        return null;
    }


    private void Update()
    {
        float timeNow = Time.time;
        // keep track of projectile life times
        for (int i = 0; i < _projectilesInPool; i++)
        {
            ProjectileHandler projectileHandler = _projectiles[i];
            if (projectileHandler._projectileObject.activeSelf && timeNow >= projectileHandler._activationTime + _projectileTimeToLive)
            {
                projectileHandler._rigidBody.velocity = Vector3.zero;
                projectileHandler._rigidBody.angularVelocity = Vector3.zero;
                projectileHandler._projectileObject.SetActive(false);
            }
        }

        if (timeNow > _lastFiredTime + _reloadTime)
        {
            // time to fire projectile
            Vector3 targetVelocity = _targetRigidbody.velocity;
            Vector3 targetPosition = _targetTransform.position;
            Vector3 projectileStartPosition = _projectileStartTransform.position;

            Vector3 interceptionPos = Vector3.zero;
            bool canFire = Util.GetInterceptPosition(projectileStartPosition, _targetTransform.position, targetVelocity, _projectileLaunchSpeed, out interceptionPos);
            if (canFire)
            {
                ProjectileHandler projectileHandler = GetAvailableProjectile();
                if (projectileHandler == null)
                    return; // no projectiles available in pool. we have to wait.

                projectileHandler._projectileObject.transform.position = projectileStartPosition;
                Vector3 dirToInterceptPos = (interceptionPos - projectileStartPosition).normalized;
                projectileHandler._rigidBody.AddForce(dirToInterceptPos * _projectileLaunchSpeed * _projectileMass, ForceMode.Impulse);
                _lastFiredTime = timeNow;
            }
        }

    }

}
