using UnityEngine;


public class BallHandler : MonoBehaviour
{
    [Header("Ball Handler properties")]
    [Tooltip("The rigid body")]
    public Rigidbody _rigidBody;

    private DebugConsoleManager _debugConsoleManager;


    private void Start()
    {
        _debugConsoleManager = FindObjectOfType<DebugConsoleManager>(true);
        _debugConsoleManager.AddConsoleCommand("DropBall", this);
        _debugConsoleManager.AddConsoleCommand("AddImpulseToBall", this);
        _debugConsoleManager.AddConsoleCommand("ShowBallRigidBodyProperties", this);
    }


    public void DropBall()
    {
        _rigidBody.isKinematic = false;
    }


    public void AddImpulseToBall(float force)
    {
        Vector3 forceVector = new Vector3(0.0f, force, 0.0f);
        _rigidBody.AddForce(forceVector, ForceMode.Impulse);
    }


    public void ShowBallRigidBodyProperties()
    {
        string rigidBodyProperties = "Mass: " + _rigidBody.mass.ToString() + ". Drag: " + _rigidBody.drag.ToString() + ". Angular drag: " + _rigidBody.angularDrag.ToString();
        _debugConsoleManager.AddToOutput(rigidBodyProperties);
    }

}
